# Tests for SecretStorage
# Author: Dmitry Shachnev, 2014-2016
# License: 3-clause BSD, see LICENSE file

# This file tests the dhcrypto module.

import unittest
from secretstorage.dhcrypto import int_to_bytes

class ConversionTest(unittest.TestCase):
	"""A test case that tests conversion functions
	between bytes and long."""

	def test_int_to_bytes(self) -> None:
		self.assertEqual(int_to_bytes(1), b'\x01')
		self.assertEqual(int_to_bytes(258), b'\x01\x02')
		self.assertEqual(int_to_bytes(1 << 64), b'\x01' + b'\x00' * 8)



#!/usr/bin/env python3

import sys, os

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath('..'))

# -- General configuration -----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.viewcode']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'SecretStorage'
copyright = '2020, Dmitry Shachnev'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = '3.2'
# The full version, including alpha/beta/rc tags.
release = '3.2.0'

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'alabaster'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
    'description': 'Python bindings to the FreeDesktop.org Secret Service API',
    'description_font_style': 'italic',
    'github_button': False,
    'sidebar_width': '260px',
}

# Custom sidebar templates, maps document names to template names.
html_sidebars = {
    '**': ['about.html', 'navigation.html', 'sourcelink.html', 'searchbox.html']
}

# -- Options for LaTeX output --------------------------------------------------

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
  ('index', 'SecretStorage.tex', 'SecretStorage Documentation',
   'Dmitry Shachnev', 'manual'),
]



# Tests for SecretStorage
# Author: Dmitry Shachnev, 2013-2018
# License: 3-clause BSD, see LICENSE file

# Various exception tests

import unittest
import secretstorage
from secretstorage.exceptions import ItemNotFoundException

class ExceptionsTest(unittest.TestCase):
	"""A test case that ensures that all SecretStorage exceptions
	are raised correctly."""

	def setUp(self) -> None:
		self.connection = secretstorage.dbus_init()
		self.collection = secretstorage.get_any_collection(self.connection)

	def tearDown(self) -> None:
		self.connection.close()

	def test_double_deleting(self) -> None:
		item = self.collection.create_item('MyItem',
			{'application': 'secretstorage-test'}, b'pa$$word')
		item.delete()
		self.assertRaises(ItemNotFoundException, item.delete)

	def test_non_existing_item(self) -> None:
		self.assertRaises(ItemNotFoundException, secretstorage.Item,
			self.connection, '/not/existing/path')

	def test_non_existing_collection(self) -> None:
		self.assertRaises(ItemNotFoundException,
			secretstorage.get_collection_by_alias,
			self.connection, 'non-existing-alias')



#!/usr/bin/env python3

from contextlib import closing
import secretstorage

with closing(secretstorage.dbus_init()) as connection:
	items = secretstorage.search_items(connection, {'application': 'secretstorage-test'})

	for item in items:
		print('Deleting item with label %r.' % item.get_label())
		item.delete()



# Tests for SecretStorage
# Author: Dmitry Shachnev, 2013-2018
# License: 3-clause BSD, see LICENSE file

# This file tests the secretstorage.Collection class.

import unittest
from secretstorage import Collection, Item
from secretstorage import dbus_init, get_any_collection, get_all_collections
from secretstorage import create_collection, get_default_collection
from secretstorage.util import BUS_NAME
from secretstorage.exceptions import ItemNotFoundException

class CollectionTest(unittest.TestCase):
	"""A test case that tests that all common methods of Collection
	class work and do not crash."""

	def setUp(self) -> None:
		self.connection = dbus_init()
		self.collection = get_any_collection(self.connection)

	def tearDown(self) -> None:
		self.connection.close()

	def test_all_collections(self) -> None:
		labels = map(Collection.get_label, get_all_collections(self.connection))
		self.assertIn(self.collection.get_label(), labels)

	def test_all_items(self) -> None:
		for item in self.collection.get_all_items():
			item.get_label()

	def test_create_empty_item(self) -> None:
		item = self.collection.create_item('', {}, b'')
		item.delete()

	def test_label(self) -> None:
		old_label = self.collection.get_label()
		self.collection.set_label('Hello!')
		self.assertEqual(self.collection.get_label(), 'Hello!')
		self.collection.set_label(old_label)
		self.assertEqual(self.collection.get_label(), old_label)


@unittest.skipIf(BUS_NAME == "org.freedesktop.secrets",
                "This test should only be run with the mocked server.")
class MockCollectionTest(unittest.TestCase):
	def setUp(self) -> None:
		self.connection = dbus_init()

	def tearDown(self) -> None:
		self.connection.close()

	def test_default_collection(self) -> None:
		collection = get_default_collection(self.connection)
		self.assertEqual(collection.get_label(), "Collection One")

	def test_deleting(self) -> None:
		collection_path = "/org/freedesktop/secrets/collection/spanish"
		collection = Collection(self.connection, collection_path)
		collection.unlock()
		collection.delete()

	def test_deleting_prompt(self) -> None:
		collection_path = "/org/freedesktop/secrets/collection/lockprompt"
		try:
			collection = Collection(self.connection, collection_path)
		except ItemNotFoundException:
			self.skipTest("This test should only be run with mock-service-lock.")
		collection.unlock()
		collection.delete()

	def test_deleting_item_prompt(self) -> None:
		item_path = "/org/freedesktop/secrets/collection/lockone/confirm"
		try:
			item = Item(self.connection, item_path)
		except ItemNotFoundException:
			self.skipTest("This test should only be run with mock-service-lock.")
		item.delete()

	def test_create_collection(self) -> None:
		collection = create_collection(self.connection, "My Label")
		self.assertEqual(collection.get_label(), "My Label")



# Tests for SecretStorage
# Author: Dmitry Shachnev, 2013-2018
# License: 3-clause BSD, see LICENSE file

# This file tests the secretstorage.Collection class.

import unittest
import time
from secretstorage import dbus_init, search_items, get_any_collection

ATTRIBUTES = {'application': 'secretstorage-test', 'attribute': 'qwerty'}
NEW_ATTRIBUTES = {'application': 'secretstorage-test',
	'newattribute': 'asdfgh'}

class ItemTest(unittest.TestCase):
	"""A test case that tests that all common methods of Item
	class work and do not crash."""

	def setUp(self) -> None:
		self.connection = dbus_init()
		self.collection = get_any_collection(self.connection)
		self.created_timestamp = time.time()
		self.item = self.collection.create_item('My item', ATTRIBUTES,
			b'pa$$word')
		self.other_item = self.collection.create_item('My item',
			ATTRIBUTES, b'', content_type='data/null')

	def tearDown(self) -> None:
		self.item.delete()
		self.other_item.delete()
		self.connection.close()

	def test_equal(self) -> None:
		self.assertEqual(self.item, self.item)
		self.assertNotEqual(self.item, self.other_item)
		self.assertEqual(self.other_item, self.other_item)

	def test_searchable(self) -> None:
		search_results = self.collection.search_items(ATTRIBUTES)
		self.assertIn(self.item, search_results)
		search_results = search_items(self.connection, ATTRIBUTES)
		self.assertIn(self.item, search_results)

	def test_item_in_all_items(self) -> None:
		all_items = self.collection.get_all_items()
		self.assertIn(self.item, all_items)

	def test_attributes(self) -> None:
		attributes = self.item.get_attributes()
		for key in ATTRIBUTES:
			self.assertEqual(ATTRIBUTES[key], attributes[key])
		self.item.set_attributes(NEW_ATTRIBUTES)
		attributes = self.item.get_attributes()
		for key in NEW_ATTRIBUTES:
			self.assertEqual(NEW_ATTRIBUTES[key], attributes[key])
		self.item.set_attributes(ATTRIBUTES)

	def test_label(self) -> None:
		self.assertEqual(self.item.get_label(), 'My item')
		self.item.set_label('Hello!')
		self.assertEqual(self.item.get_label(), 'Hello!')

	def test_secret(self) -> None:
		self.assertEqual(self.item.get_secret(), b'pa$$word')
		self.item.set_secret(b'newpa$$word')
		self.assertIsInstance(self.item.get_secret(), bytes)
		self.assertEqual(self.item.get_secret(), b'newpa$$word')
		self.assertEqual(self.other_item.get_secret(), b'')

	def test_secret_wrong_type(self) -> None:
		# string passwords are encoded as bytes
		self.item.set_secret('test тест')  # type: ignore
		self.assertEqual(self.item.get_secret(),
		                 'test тест'.encode('utf-8'))
		# other types are not allowed
		with self.assertRaises(TypeError):
			self.item.set_secret(None)  # type: ignore

	def test_secret_content_type(self) -> None:
		self.assertEqual(self.item.get_secret_content_type(), 'text/plain')
		# The check below fails in gnome-keyring because it doesn't really
		# support content types.
		#self.assertEqual(self.other_item.get_secret_content_type(), 'data/null')

	def test_modified(self) -> None:
		now = time.time()
		modified = self.item.get_modified()
		self.assertAlmostEqual(now, modified, places=-1)

	def test_created(self) -> None:
		created = self.item.get_created()
		self.assertAlmostEqual(self.created_timestamp, created, places=-1)

	def test_unlock(self) -> None:
		self.item.unlock()



# Tests for SecretStorage
# Author: Dmitry Shachnev, 2018
# License: 3-clause BSD, see LICENSE file

import unittest

from secretstorage import dbus_init, Collection
from secretstorage.util import BUS_NAME
from secretstorage.exceptions import LockedException


@unittest.skipIf(BUS_NAME == "org.freedesktop.secrets",
                 "This test should only be run with the mocked server.")
class LockingUnlockingTest(unittest.TestCase):
	def setUp(self) -> None:
		self.connection = dbus_init()
		collection_path = "/org/freedesktop/secrets/collection/english"
		self.collection = Collection(self.connection, collection_path)

	def tearDown(self) -> None:
		self.connection.close()

	def test_lock_unlock(self) -> None:
		self.assertFalse(self.collection.is_locked())
		self.collection.lock()
		self.assertTrue(self.collection.is_locked())
		self.assertRaises(LockedException, self.collection.ensure_not_locked)
		item, = self.collection.search_items({"number": "1"})
		self.assertRaises(LockedException, item.ensure_not_locked)
		self.assertIs(self.collection.unlock(), False)
		self.assertFalse(self.collection.is_locked())
		self.collection.ensure_not_locked()



# Tests for SecretStorage
# Author: Dmitry Shachnev, 2019
# License: 3-clause BSD, see LICENSE file

# This file tests using secretstorage.dbus_init() function
# together with contextlib.closing context manager.

from contextlib import closing
import unittest
from secretstorage import check_service_availability, dbus_init
from secretstorage.collection import get_any_collection


class ContextManagerTest(unittest.TestCase):
	"""``dbus_init()`` should work fine with ``contextlib.closing``
	context manager."""

	def test_closing_context_manager(self) -> None:
		with closing(dbus_init()) as connection:
			self.assertTrue(check_service_availability(connection))
			collection = get_any_collection(connection)
			self.assertIsNotNone(collection)
			label = collection.get_label()
			self.assertIsNotNone(label)



#!/usr/bin/env python

import os.path
import sys
import subprocess
import unittest

tests_dir = os.path.dirname(__file__)
sys.path.insert(0, os.path.dirname(tests_dir))

import secretstorage

if __name__ == '__main__':
	major, minor, patch = sys.version_info[:3]
	print('Running with Python %d.%d.%d (SecretStorage from %s)' %
	      (major, minor, patch, os.path.dirname(secretstorage.__file__)))
	mock = None
	if len(sys.argv) > 1 and os.path.isfile(sys.argv[1]):
		mock = subprocess.Popen(('/usr/bin/python3', sys.argv[1],),
		                        stdout=subprocess.PIPE,
		                        universal_newlines=True)
		assert mock.stdout is not None  # for mypy
		bus_name = mock.stdout.readline().rstrip()
		secretstorage.util.BUS_NAME = bus_name
		print('Bus name set to %r' % secretstorage.util.BUS_NAME)
	loader = unittest.TestLoader()
	runner = unittest.TextTestRunner(verbosity=2)
	result = runner.run(loader.discover(tests_dir))
	if mock is not None:
		mock.terminate()
	sys.exit(not result.wasSuccessful())
